@@ -52,13 +52,16 @@ correctly on all compute nodes within the cluster.
 This strategy assumes it is possible to live migrate any VM from
 an active compute node to any other active compute node.
 """
+import datetime
 
+from oslo_config import cfg
 from oslo_log import log
 import six
 
-from watcher._i18n import _, _LE, _LI
+from watcher._i18n import _
 from watcher.common import exception
 from watcher.datasource import ceilometer as ceil
+from watcher.datasource import gnocchi as gnoc
 from watcher.decision_engine.model import element
 from watcher.decision_engine.strategy.strategies import base
 
@@ -68,12 +71,33 @@ LOG = log.getLogger(__name__)
 class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
     """VM Workload Consolidation Strategy"""
 
+    HOST_CPU_USAGE_METRIC_NAME = 'compute.node.cpu.percent'
+    INSTANCE_CPU_USAGE_METRIC_NAME = 'cpu_util'
+
+    METRIC_NAMES = dict(
+        ceilometer=dict(
+            cpu_util_metric='cpu_util',
+            ram_util_metric='memory.resident',
+            ram_alloc_metric='memory',
+            disk_alloc_metric='disk.root.size'),
+        gnocchi=dict(
+            cpu_util_metric='cpu_util',
+            ram_util_metric='memory.resident',
+            ram_alloc_metric='memory',
+            disk_alloc_metric='disk.root.size'),
+    )
+
+    MIGRATION = "migrate"
+    CHANGE_NOVA_SERVICE_STATE = "change_nova_service_state"
+
     def __init__(self, config, osc=None):
         super(VMWorkloadConsolidation, self).__init__(config, osc)
         self._ceilometer = None
+        self._gnocchi = None
         self.number_of_migrations = 0
         self.number_of_released_nodes = 0
-        self.ceilometer_instance_data_cache = dict()
+        # self.ceilometer_instance_data_cache = dict()
+        self.datasource_instance_data_cache = dict()
 
     @classmethod
     def get_name(cls):
@@ -87,6 +111,10 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
     def get_translatable_display_name(cls):
         return "VM Workload Consolidation Strategy"
 
+    @property
+    def period(self):
+        return self.input_parameters.get('period', 3600)
+
     @property
     def ceilometer(self):
         if self._ceilometer is None:
@@ -97,20 +125,80 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
     def ceilometer(self, ceilometer):
         self._ceilometer = ceilometer
 
-    def get_state_str(self, state):
-        """Get resource state in string format.
+    @property
+    def gnocchi(self):
+        if self._gnocchi is None:
+            self._gnocchi = gnoc.GnocchiHelper(osc=self.osc)
+        return self._gnocchi
 
-        :param state: resource state of unknown type
+    @gnocchi.setter
+    def gnocchi(self, gnocchi):
+        self._gnocchi = gnocchi
+
+    @property
+    def granularity(self):
+        return self.input_parameters.get('granularity', 300)
+
+    @classmethod
+    def get_schema(cls):
+        # Mandatory default setting for each element
+        return {
+            "properties": {
+                "period": {
+                    "description": "The time interval in seconds for "
+                                   "getting statistic aggregation",
+                    "type": "number",
+                    "default": 3600
+                },
+                "granularity": {
+                    "description": "The time between two measures in an "
+                                   "aggregated timeseries of a metric.",
+                    "type": "number",
+                    "default": 300
+                },
+            }
+        }
+
+    @classmethod
+    def get_config_opts(cls):
+        return [
+            cfg.StrOpt(
+                "datasource",
+                help="Data source to use in order to query the needed metrics",
+                default="ceilometer",
+                choices=["ceilometer", "gnocchi"])
+        ]
+
+    def get_instance_state_str(self, instance):
+        """Get instance state in string format.
+
+        :param instance:
         """
-        if isinstance(state, six.string_types):
-            return state
-        elif isinstance(state, (element.InstanceState, element.ServiceState)):
-            return state.value
+        if isinstance(instance.state, six.string_types):
+            return instance.state
+        elif isinstance(instance.state, element.InstanceState):
+            return instance.state.value
         else:
-            LOG.error(_LE('Unexpexted resource state type, '
-                          'state=%(state)s, state_type=%(st)s.') % dict(
-                              state=state,
-                              st=type(state)))
+            LOG.error('Unexpected instance state type, '
+                      'state=%(state)s, state_type=%(st)s.' %
+                      dict(state=instance.state,
+                           st=type(instance.state)))
+            raise exception.WatcherException
+
+    def get_node_status_str(self, node):
+        """Get node status in string format.
+
+        :param node:
+        """
+        if isinstance(node.status, six.string_types):
+            return node.status
+        elif isinstance(node.status, element.ServiceState):
+            return node.status.value
+        else:
+            LOG.error('Unexpected node status type, '
+                      'status=%(status)s, status_type=%(st)s.' %
+                      dict(status=node.status,
+                           st=type(node.status)))
             raise exception.WatcherException
 
     def add_action_enable_compute_node(self, node):
@@ -121,7 +209,7 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
         """
         params = {'state': element.ServiceState.ENABLED.value}
         self.solution.add_action(
-            action_type='change_nova_service_state',
+            action_type=self.CHANGE_NOVA_SERVICE_STATE,
             resource_id=node.uuid,
             input_parameters=params)
         self.number_of_released_nodes -= 1
@@ -134,7 +222,7 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
         """
         params = {'state': element.ServiceState.DISABLED.value}
         self.solution.add_action(
-            action_type='change_nova_service_state',
+            action_type=self.CHANGE_NOVA_SERVICE_STATE,
             resource_id=node.uuid,
             input_parameters=params)
         self.number_of_released_nodes += 1
@@ -147,23 +235,27 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
         :param destination_node: node object
         :return: None
         """
-        instance_state_str = self.get_state_str(instance.state)
+        instance_state_str = self.get_instance_state_str(instance)
         if instance_state_str != element.InstanceState.ACTIVE.value:
-            # Watcher curently only supports live VM migration and block live
+            # Watcher currently only supports live VM migration and block live
             # VM migration which both requires migrated VM to be active.
             # When supported, the cold migration may be used as a fallback
             # migration mechanism to move non active VMs.
             LOG.error(
-                _LE('Cannot live migrate: instance_uuid=%(instance_uuid)s, '
-                    'state=%(instance_state)s.') % dict(
-                        instance_uuid=instance.uuid,
-                        instance_state=instance_state_str))
+                'Cannot live migrate: instance_uuid=%(instance_uuid)s, '
+                'state=%(instance_state)s.' % dict(
+                    instance_uuid=instance.uuid,
+                    instance_state=instance_state_str))
             return
 
         migration_type = 'live'
 
-        destination_node_state_str = self.get_state_str(destination_node.state)
-        if destination_node_state_str == element.ServiceState.DISABLED.value:
+        # Here will makes repeated actions to enable the same compute node,
+        # when migrating VMs to the destination node which is disabled.
+        # Whether should we remove the same actions in the solution???
+        destination_node_status_str = self.get_node_status_str(
+            destination_node)
+        if destination_node_status_str == element.ServiceState.DISABLED.value:
             self.add_action_enable_compute_node(destination_node)
 
         if self.compute_model.migrate_instance(
@@ -171,13 +263,13 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
             params = {'migration_type': migration_type,
                       'source_node': source_node.uuid,
                       'destination_node': destination_node.uuid}
-            self.solution.add_action(action_type='migrate',
+            self.solution.add_action(action_type=self.MIGRATION,
                                      resource_id=instance.uuid,
                                      input_parameters=params)
             self.number_of_migrations += 1
 
     def disable_unused_nodes(self):
-        """Generate actions for disablity of unused nodes.
+        """Generate actions for disabling unused nodes.
 
         :return: None
         """
@@ -187,62 +279,105 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
                     element.ServiceState.DISABLED.value):
                 self.add_action_disable_node(node)
 
-    def get_instance_utilization(self, instance,
-                                 period=3600, aggr='avg'):
+    def get_instance_utilization(self, instance):
         """Collect cpu, ram and disk utilization statistics of a VM.
 
         :param instance: instance object
-        :param period: seconds
         :param aggr: string
         :return: dict(cpu(number of vcpus used), ram(MB used), disk(B used))
         """
-        if instance.uuid in self.ceilometer_instance_data_cache.keys():
-            return self.ceilometer_instance_data_cache.get(instance.uuid)
+        instance_cpu_util = None
+        instance_ram_util = None
+        instance_disk_util = None
 
-        cpu_util_metric = 'cpu_util'
-        ram_util_metric = 'memory.usage'
+        if instance.uuid in self.datasource_instance_data_cache.keys():
+            return self.datasource_instance_data_cache.get(instance.uuid)
 
-        ram_alloc_metric = 'memory'
-        disk_alloc_metric = 'disk.root.size'
-        instance_cpu_util = self.ceilometer.statistic_aggregation(
-            resource_id=instance.uuid, meter_name=cpu_util_metric,
-            period=period, aggregate=aggr)
+        cpu_util_metric = self.METRIC_NAMES[
+            self.config.datasource]['cpu_util_metric']
+        ram_util_metric = self.METRIC_NAMES[
+            self.config.datasource]['ram_util_metric']
+        ram_alloc_metric = self.METRIC_NAMES[
+            self.config.datasource]['ram_alloc_metric']
+        disk_alloc_metric = self.METRIC_NAMES[
+            self.config.datasource]['disk_alloc_metric']
 
+        if self.config.datasource == "ceilometer":
+            instance_cpu_util = self.ceilometer.statistic_aggregation(
+                resource_id=instance.uuid, meter_name=cpu_util_metric,
+                period=self.period, aggregate='avg')
+            instance_ram_util = self.ceilometer.statistic_aggregation(
+                resource_id=instance.uuid, meter_name=ram_util_metric,
+                period=self.period, aggregate='avg')
+            if not instance_ram_util:
+                instance_ram_util = self.ceilometer.statistic_aggregation(
+                    resource_id=instance.uuid, meter_name=ram_alloc_metric,
+                    period=self.period, aggregate='avg')
+            instance_disk_util = self.ceilometer.statistic_aggregation(
+                resource_id=instance.uuid, meter_name=disk_alloc_metric,
+                period=self.period, aggregate='avg')
+        elif self.config.datasource == "gnocchi":
+            stop_time = datetime.datetime.utcnow()
+            start_time = stop_time - datetime.timedelta(
+                seconds=int(self.period))
+            instance_cpu_util = self.gnocchi.statistic_aggregation(
+                resource_id=instance.uuid,
+                metric=cpu_util_metric,
+                granularity=self.granularity,
+                start_time=start_time,
+                stop_time=stop_time,
+                aggregation='mean'
+            )
+            instance_ram_util = self.gnocchi.statistic_aggregation(
+                resource_id=instance.uuid,
+                metric=ram_util_metric,
+                granularity=self.granularity,
+                start_time=start_time,
+                stop_time=stop_time,
+                aggregation='mean'
+            )
+            if not instance_ram_util:
+                instance_ram_util = self.gnocchi.statistic_aggregation(
+                    resource_id=instance.uuid,
+                    metric=ram_alloc_metric,
+                    granularity=self.granularity,
+                    start_time=start_time,
+                    stop_time=stop_time,
+                    aggregation='mean'
+                )
+            instance_disk_util = self.gnocchi.statistic_aggregation(
+                resource_id=instance.uuid,
+                metric=disk_alloc_metric,
+                granularity=self.granularity,
+                start_time=start_time,
+                stop_time=stop_time,
+                aggregation='mean'
+            )
         if instance_cpu_util:
             total_cpu_utilization = (
                 instance.vcpus * (instance_cpu_util / 100.0))
         else:
             total_cpu_utilization = instance.vcpus
 
-        instance_ram_util = self.ceilometer.statistic_aggregation(
-            resource_id=instance.uuid, meter_name=ram_util_metric,
-            period=period, aggregate=aggr)
-
         if not instance_ram_util:
-            instance_ram_util = self.ceilometer.statistic_aggregation(
-                resource_id=instance.uuid, meter_name=ram_alloc_metric,
-                period=period, aggregate=aggr)
+            instance_ram_util = instance.memory
+            LOG.warning('No values returned by %s for memory.resident, '
+                        'use instance flavor ram value', instance.uuid)
 
-        instance_disk_util = self.ceilometer.statistic_aggregation(
-            resource_id=instance.uuid, meter_name=disk_alloc_metric,
-            period=period, aggregate=aggr)
+        if not instance_disk_util:
+            instance_disk_util = instance.disk
+            LOG.warning('No values returned by %s for disk.root.size, '
+                        'use instance flavor disk value', instance.uuid)
 
-        if not instance_ram_util or not instance_disk_util:
-            LOG.error(
-                _LE('No values returned by %s for memory.usage '
-                    'or disk.root.size'), instance.uuid)
-            raise exception.NoDataFound
-
-        self.ceilometer_instance_data_cache[instance.uuid] = dict(
+        self.datasource_instance_data_cache[instance.uuid] = dict(
             cpu=total_cpu_utilization, ram=instance_ram_util,
             disk=instance_disk_util)
-        return self.ceilometer_instance_data_cache.get(instance.uuid)
+        return self.datasource_instance_data_cache.get(instance.uuid)
 
-    def get_node_utilization(self, node, period=3600, aggr='avg'):
+    def get_node_utilization(self, node):
         """Collect cpu, ram and disk utilization statistics of a node.
 
         :param node: node object
-        :param period: seconds
         :param aggr: string
         :return: dict(cpu(number of cores used), ram(MB used), disk(B used))
         """
@@ -252,7 +387,7 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
         node_cpu_util = 0
         for instance in node_instances:
             instance_util = self.get_instance_utilization(
-                instance, period, aggr)
+                instance)
             node_cpu_util += instance_util['cpu']
             node_ram_util += instance_util['ram']
             node_disk_util += instance_util['disk']
@@ -291,8 +426,8 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
         rcu = {}
         counters = {}
         for node in nodes:
-            node_state_str = self.get_state_str(node.state)
-            if node_state_str == element.ServiceState.ENABLED.value:
+            node_status_str = self.get_node_status_str(node)
+            if node_status_str == element.ServiceState.ENABLED.value:
                 rhu = self.get_relative_node_utilization(node)
                 for k in rhu.keys():
                     if k not in rcu:
@@ -357,7 +492,7 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
         """
         migrate_actions = (
             a for a in self.solution.actions if a[
-                'action_type'] == 'migrate')
+                'action_type'] == self.MIGRATION)
         instance_to_be_migrated = (
             a['input_parameters']['resource_id'] for a in migrate_actions)
         instance_uuids = list(set(instance_to_be_migrated))
@@ -387,11 +522,11 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
         Offload phase performing first-fit based bin packing to offload
         overloaded nodes. This is done in a fashion of moving
         the least CPU utilized VM first as live migration these
-        generaly causes less troubles. This phase results in a cluster
+        generally causes less troubles. This phase results in a cluster
         with no overloaded nodes.
         * This phase is be able to enable disabled nodes (if needed
         and any available) in the case of the resource capacity provided by
-        active nodes is not able to accomodate all the load.
+        active nodes is not able to accommodate all the load.
         As the offload phase is later followed by the consolidation phase,
         the node enabler in this phase doesn't necessarily results
         in more enabled nodes in the final solution.
@@ -424,9 +559,9 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
         Consolidation phase performing first-fit based bin packing.
         First, nodes with the lowest cpu utilization are consolidated
         by moving their load to nodes with the highest cpu utilization
-        which can accomodate the load. In this phase the most cpu utilizied
-        VMs are prioritizied as their load is more difficult to accomodate
-        in the system than less cpu utilizied VMs which can be later used
+        which can accommodate the load. In this phase the most cpu utilized
+        VMs are prioritized as their load is more difficult to accommodate
+        in the system than less cpu utilized VMs which can be later used
         to fill smaller CPU capacity gaps.
 
         :param cc: dictionary containing resource capacity coefficients
@@ -475,7 +610,7 @@ class VMWorkloadConsolidation(base.ServerConsolidationBaseStrategy):
 
         :param original_model: root_model object
         """
-        LOG.info(_LI('Executing Smart Strategy'))
+        LOG.info('Executing Smart Strategy')
         rcu = self.get_relative_cluster_utilization()
 
         cc = {'cpu': 1.0, 'ram': 1.0, 'disk': 1.0}
